set feedback off
drop table salary_history;
drop table job_history;
drop table children;
drop table employees;
drop cluster employees;
DROP TYPE employee_rec;
DROP TYPE job_history_tab;
DROP TYPE job_history_rec;
DROP TYPE salary_history_tab;
DROP TYPE salary_history_rec;
DROP TYPE children_tab;
DROP TYPE children_rec;
set feedback on
set echo on
CREATE CLUSTER employees (employee# INTEGER) hashkeys 1000;
pause
CREATE TABLE employees
(
  employee# INTEGER NOT NULL,
  name VARCHAR2(16),
  birth_date DATE,
  CONSTRAINT employees_pk PRIMARY KEY (employee#)
)
CLUSTER employees (employee#);
pause
CREATE TABLE job_history
(
  employee# INTEGER NOT NULL,
  job_date DATE NOT NULL,
  title VARCHAR2(16),
  CONSTRAINT job_history_pk PRIMARY KEY (employee#, job_date),
  CONSTRAINT job_history_fk1 FOREIGN KEY (employee#) REFERENCES employees
)
CLUSTER employees (employee#);
pause
CREATE TABLE salary_history
(
  employee# INTEGER NOT NULL,
  job_date DATE NOT NULL,
  salary_date DATE NOT NULL,
  salary NUMBER,
  CONSTRAINT salary_history_pk PRIMARY KEY (employee#, job_date, salary_date),
  CONSTRAINT salary_history_fk1 FOREIGN KEY (employee#) REFERENCES employees,
  CONSTRAINT salary_history_fk2 FOREIGN KEY (employee#, job_date) REFERENCES job_history
)
CLUSTER employees (employee#);
pause
CREATE TABLE children
(
  employee# INTEGER NOT NULL,
  child_name VARCHAR2(16) NOT NULL,
  birth_date DATE,
  CONSTRAINT children_pk PRIMARY KEY (employee#, child_name),
  CONSTRAINT children_fk1 FOREIGN KEY (employee#) REFERENCES employees
)
CLUSTER employees (employee#);
pause
INSERT INTO employees VALUES (1, 'IGNATIUS', '01-JAN-1970');
pause
INSERT INTO children VALUES (1, 'INIGA', '01-JAN-2001');
pause
INSERT INTO children VALUES (1, 'INIGO', '01-JAN-2002');
pause
INSERT INTO job_history VALUES (1, '01-JAN-1991', 'PROGRAMMER');
pause
INSERT INTO job_history VALUES (1, '01-JAN-1992', 'DATABASE ADMIN');
pause
INSERT INTO salary_history VALUES (1, '01-JAN-1991', '1-FEB-1991', 1000);
pause
INSERT INTO salary_history VALUES (1, '01-JAN-1991', '1-MAR-1991', 1000);
pause
INSERT INTO salary_history VALUES (1, '01-JAN-1992', '1-FEB-1992', 2000);
pause
INSERT INTO salary_history VALUES (1, '01-JAN-1992', '1-MAR-1992', 2000);
pause
SELECT DISTINCT DBMS_ROWID.ROWID_BLOCK_NUMBER(rowid) AS block_number FROM employees where employee# = 1;
pause
SELECT DISTINCT DBMS_ROWID.ROWID_BLOCK_NUMBER(rowid) AS block_number FROM children where employee# = 1;
pause
SELECT DISTINCT DBMS_ROWID.ROWID_BLOCK_NUMBER(rowid) AS block_number FROM job_history where employee# = 1;
pause
SELECT DISTINCT DBMS_ROWID.ROWID_BLOCK_NUMBER(rowid) AS block_number FROM salary_history where employee# = 1;
pause
CREATE OR REPLACE TYPE children_rec AS OBJECT (child_name VARCHAR2(16), birth_date DATE)
/
pause
CREATE OR REPLACE TYPE children_tab AS TABLE OF children_rec
/
pause
CREATE OR REPLACE TYPE salary_history_rec AS OBJECT (salary_date DATE, salary NUMBER)
/
pause
CREATE OR REPLACE TYPE salary_history_tab AS TABLE OF salary_history_rec
/
pause
CREATE OR REPLACE TYPE job_history_rec AS OBJECT (job_date DATE, title VARCHAR2(16), salary_history SALARY_HISTORY_TAB)
/
pause
CREATE OR REPLACE TYPE job_history_tab AS TABLE of job_history_rec
/
pause
CREATE OR REPLACE TYPE employee_rec AS OBJECT (employee# INTEGER, name VARCHAR2(16), birth_date DATE, children CHILDREN_TAB, job_history JOB_HISTORY_TAB)
/
pause
create or replace view employees_view as
SELECT
  employee#,
  name,
  birth_date,
  CAST
  (
    MULTISET
    (
      SELECT
        child_name,
        birth_date
      FROM children
      WHERE employee#=e.employee#
    )
    AS children_tab
  ) children,
  CAST
  (
    MULTISET
    (
      SELECT
        job_date,
        title,
        CAST
        (
          MULTISET
          (
            SELECT salary_date, salary
            FROM salary_history
            WHERE employee#=e.employee#
            AND job_date=jh.job_date
          )
          AS salary_history_tab
        ) salary_history
      FROM job_history jh
      WHERE employee#=e.employee#
    )
    AS job_history_tab
  ) job_history
FROM employees e;
pause
alter session set "_rowsource_execution_statistics"=true;
set linesize 132
set pagesize 60
SELECT * FROM employees_view WHERE employee# = 1;
pause
SELECT * FROM TABLE(dbms_xplan.display_cursor(null, null, 'TYPICAL IOSTATS LAST'));
pause
CREATE OR REPLACE TRIGGER employees_view_insert
INSTEAD OF INSERT ON employees_view
REFERENCING NEW AS n
FOR EACH ROW
DECLARE
  i NUMBER;
BEGIN
  INSERT INTO employees
  VALUES
  (
    :n.employee#,
    :n.name,
    :n.birth_date
  );

  FOR i IN :n.children.FIRST .. :n.children.LAST
  LOOP
    INSERT INTO children
    VALUES
    (
      :n.employee#,
      :n.children(i).child_name,
      :n.children(i).birth_date
    );
  END LOOP;

  FOR i IN :n.job_history.FIRST .. :n.job_history.LAST
  LOOP
    INSERT INTO job_history VALUES
    (
      :n.employee#,
      :n.job_history(i).job_date,
      :n.job_history(i).title
    );
    FOR j IN :n.job_history(i).salary_history.FIRST .. :n.job_history(i).salary_history.LAST
    LOOP
      INSERT INTO salary_history
      VALUES
      (
        :n.employee#,
        :n.job_history(i).job_date,
        :n.job_history(i).salary_history(j).salary_date,
        :n.job_history(i).salary_history(j).salary
       );
    END LOOP;
  END LOOP;
END;
/
pause
INSERT INTO employees_view
VALUES
(
  2,
  'YGNACIO',
  '01-JAN-70',
  CHILDREN_TAB
  (
    CHILDREN_REC('INIGA', '01-JAN-01'),
    CHILDREN_REC('INIGO', '01-JAN-02')
  ),
  JOB_HISTORY_TAB
  (
    JOB_HISTORY_REC
    (
      '01-JAN-91',
      'PROGRAMMER',
      SALARY_HISTORY_TAB
      (
        SALARY_HISTORY_REC('01-FEB-91', 1000),
        SALARY_HISTORY_REC('01-MAR-91', 1000)
      )
    ), 
    JOB_HISTORY_REC
    (
      '01-JAN-92',
      'DATABASE ADMIN',
      SALARY_HISTORY_TAB
      (
        SALARY_HISTORY_REC('01-FEB-92', 2000),
        SALARY_HISTORY_REC('01-MAR-92', 2000)
      )
    )
  )
);
pause
SELECT * FROM employees_view WHERE employee# = 2;
pause
exit